%********************************************************************
%   Post Process for fixes
%   version 1.0             30/05/2018
%
%  Script to extract the FIXATIONS after an Event  
%  Onset sample is derived from the timestamp entries and the hits in
%  the AOI regions are then accumulated during this time
%
%  Assumes 16.667ms sample time on the raw data (120Hz), reporting a
%  fixation as > 2 samples (32ms)
%
%  Requires GazeData.csv & StimulusInfo.csv to be imported first as COLUMNS
%  (just drag these over to workspace and import)
%
%  Requires that user enters the subject ID when requested
%
%  Data is output in the following format as ConsiseFixationReport : -
%
%       subjectID  ( as manually entered on running the script)
%       trial
%       fixation_number (within session) 
%       trialOnset time (within session)
%       AOI side for fixation ( 0 = LEFT, 1 = RIGHT)
%       fixation (within trial)
%       image code (from coding table)
%       sound code (from coding table)
%       fixation entry time refered to trial onset (ms)
%       fixation exit  time refered to trial onset (ms)
%       duration of fixation (ms)
%
%********************************************************************


AOI_left = [0.1, 0.3, 0.4, 0.7];
AOI_right = [0.6, 0.3, 0.9, 0.7];
fixCriteria = 35;           % pixel value for fixation instance

FixationReport = [];       % Fixation report array
entry = 0;                 % initialise the Fixation report entry index value for this session
AOI_side = 3;
nextTrial = false;

%********************************************************************
% Enter the subject number on the keyboard
%********************************************************************
SUBJECT = input('Subject number: ');
fprintf('\n');

%********************************************************************
% find the trial onset sample number for each trial
%********************************************************************
maxTrial = size(VarName9);
for trial = 1 : maxTrial(1)   
   totalSamples = size(RemotetoLocalTimestamp);
   for sampleNo = 1 : totalSamples(1)
       gazeDataSampleTime = RemotetoLocalTimestamp(sampleNo);
       if  gazeDataSampleTime >=  VarName9(trial) * 1000000 % *1000000 required here
           trialOnset(trial) = sampleNo;
           nextTrial = true;
           break
       end
       if nextTrial
           break
       end
   end
   nextTrial = false;   
end
fprintf('finished gathering Onset times \n');


fixation_number = 0;
fixCriteria = 35;

       fixation_entry_time = 0;
       fixation_exit_time = 0;
       active_fixation_Left = false;
       active_fixation_Right = false;
       
       AOI_Left_Total_Duration = 0;
       AOI_Right_Total_Duration = 0;
       
       
sampleCount = 0;
%********************************************************************
% for each trial
%******************************************************************** 
for trial = 1 : maxTrial(1)
   
   fixation_entry_time = 0;
   fixation_exit_time = 0;
   trial_fixation = 0;
   
   %********************************************************************
   % for duration of image collect hits to each AOI
   %********************************************************************  
   lastSample = trialOnset(trial) + 280;                                  % equates to 4.5 seconds
   for sampleNo = trialOnset(trial) : lastSample
       report_sample = false;           % assume this for this sample
       eTag = 0;
       
   %********************************************************************
   % check for Left AOI
   %******************************************************************** 
       if (GazePoint2d_Leftx(sampleNo) >= AOI_left(1)) && (GazePoint2d_Leftx(sampleNo) <= AOI_left(3)) && (GazePoint2d_Lefty(sampleNo) >= AOI_left(2)) && (GazePoint2d_Lefty(sampleNo) <= AOI_left(4))
               AOI_side = 0;                                              % indicates a LEFT AOI
               AOI_Left_Total_Duration = AOI_Left_Total_Duration + 1;     % Total LEFT looking time accumulator

               if ~active_fixation_Left                                   % this a fresh fixation LEFT          
                   fixation_number = fixation_number + 1;                 % set the fixation ID for fresh fixation
                   trial_fixation = trial_fixation + 1;
                   fixation_entry_time = sampleNo;                        % record the start of this fixation
                   eTag = 1;
                   active_fixation_Left = true;                           % acknowledge a new fixation has occured in Left AOI during required timeframe
                   report_sample = true;
                   lastGazeX = GazePoint2d_Leftx(sampleNo);               % initial Gaze point of fixation X and Y coordinates
                   lastGazeY = GazePoint2d_Lefty(sampleNo);   
               elseif ((GazePoint2d_Leftx(sampleNo) > (lastGazeX + fixCriteria)) || (GazePoint2d_Leftx(sampleNo) < (lastGazeX - fixCriteria)) || (GazePoint2d_Lefty(sampleNo) > (lastGazeY + fixCriteria)) || (GazePoint2d_Lefty(sampleNo) < (lastGazeY - fixCriteria)))
                   fixation_exit_time = sampleNo;                         %  report the exit time
                   active_fixation_Left = false;                          %  indicate that fixation has ended
                   report_sample = true;
                   eTag = 2;           
               elseif sampleNo == lastSample
                   fixation_exit_time = sampleNo;                          %  report the exit time
                   active_fixation_Left = false;                          %  indicate that fixation has ended
                   report_sample = true;
                   eTag = 6;
               end
               
   %********************************************************************
   % check for Right AOI
   %********************************************************************
        elseif (GazePoint2d_Leftx(sampleNo) >= AOI_right(1)) && (GazePoint2d_Leftx(sampleNo) <= AOI_right(3)) && (GazePoint2d_Lefty(sampleNo) >= AOI_right(2)) && (GazePoint2d_Lefty(sampleNo) <= AOI_right(4))   
           AOI_side = 1;                                              % indicates a RIGHT AOI
           AOI_Right_Total_Duration = AOI_Right_Total_Duration + 1;   % Total LEFT looking time accumulator
           
           if ~active_fixation_Right                                  % this a fresh fixation RIGHT
               fixation_number = fixation_number + 1;                 % set the fixation ID for fresh fixation
               trial_fixation = trial_fixation + 1;
               fixation_entry_time = sampleNo;                        % record the start of this fixation
               eTag = 3;  
               active_fixation_Right = true;                          % acknowledge a new fixation has occured in Left AOI during required timeframe
               report_sample = true;
               lastGazeX = GazePoint2d_Leftx(sampleNo);               % initial Gaze point of fixation X and Y coordinates
               lastGazeY = GazePoint2d_Lefty(sampleNo);     
           elseif ((GazePoint2d_Leftx(sampleNo) > (lastGazeX + fixCriteria)) || (GazePoint2d_Leftx(sampleNo) < (lastGazeX - fixCriteria)) || (GazePoint2d_Lefty(sampleNo) > (lastGazeY + fixCriteria)) || (GazePoint2d_Lefty(sampleNo) < (lastGazeY - fixCriteria)))
                   fixation_exit_time = sampleNo;                          %  report the exit time
                   active_fixation_Right = false;                          %  indicate that fixation has ended
                   report_sample = true;
                   eTag = 4;
           elseif sampleNo == lastSample
                   fixation_exit_time = sampleNo;                          %  report the exit time
                   active_fixation_Right = false;                          %  indicate that fixation has ended
                   report_sample = true;
                   eTag = 7;
           end
           
   %********************************************************************
   % check for Gaze outside either AOI
   %********************************************************************                   
       else
           if active_fixation_Right  || active_fixation_Left  
                   fixation_exit_time = sampleNo;                          %  report the exit time for fixation
                   report_sample = true;
                   eTag = 5;
           elseif  sampleNo == lastSample
                   fixation_exit_time = sampleNo;                          %  report the exit time
                   report_sample = true;
                   eTag = 8;                 
           end 
           active_fixation_Right = false;                          %  indicate that fixation has ended on Left or Right by default
           active_fixation_Left = false;
       end
       
   %********************************************************************
   % Report anything for this sample pass in the FixationReport
   %********************************************************************        
       if report_sample
           if (eTag == 1 || eTag == 3)
               entry = entry + 1;
               FixationReport(entry, 1) = trial;
               FixationReport(entry, 2) = fixation_number;                  % session fixation number
               FixationReport(entry, 3) = trialOnset(trial);                % report start of trial time within session
               FixationReport(entry, 4) = fixation_entry_time;              % report fixation entry time
               FixationReport(entry, 5) = 0;                                % fixation_exit_time;
               FixationReport(entry, 6) = 0;                                % padding
               FixationReport(entry, 7) = AOI_side;                         % Left or Right AOI id
               FixationReport(entry, 8)  = 0;                               % fixation exit tag id
               FixationReport(entry, 9)  = 0;
               FixationReport(entry, 10) = 0;
               FixationReport(entry, 11) = 0;
               FixationReport(entry, 12) = 0;
               FixationReport(entry, 13) = 0;                               % Image code
               FixationReport(entry, 14) = 0;                               % Sound code
               FixationReport(entry, 15) = 0;                               % NULL
               FixationReport(entry, 16) = 0;
               FixationReport(entry, 17) = 0;
               FixationReport(entry, 18) = 0;
               FixationReport(entry, 19) = 0;
               
           elseif (eTag == 2 || eTag == 4 || eTag == 5 || eTag == 6 || eTag == 7 || eTag == 8) 
               FixationReport(entry, 5) = fixation_exit_time;                                                   % fixation_exit_time;
               FixationReport(entry, 6) = fixation_exit_time - fixation_entry_time;                             % duration of fixation
               FixationReport(entry, 8) = eTag;                                                                 % fixation exit tag id
               FixationReport(entry, 9) = trial_fixation;                                                       % fixation within trial
               FixationReport(entry, 10) = FixationReport(entry, 4) - FixationReport(entry, 3) ;    % fixation entry time refered to trial onset (samples)
               FixationReport(entry, 11) = fixation_exit_time       - FixationReport(entry, 3);    % fixation exit  time refered to trial onset (samples)
               FixationReport(entry, 12) = FixationReport(entry, 11) - FixationReport(entry, 10);  % duration of fixation (samples)
               FixationReport(entry, 13) = VarName4(trial);                                                     % Image code
               FixationReport(entry, 14) = VarName5(trial);                                                     % Sound code
               FixationReport(entry, 15) = 0;                                                                   % NULL
               FixationReport(entry, 16) = round(FixationReport(entry, 10) * 16.66667) ;                   % fixation entry time refered to trial onset (ms)
               FixationReport(entry, 17) = round(FixationReport(entry, 11) * 16.66667) ;                   % fixation exit  time refered to trial onset (ms)
               FixationReport(entry, 18) = round(FixationReport(entry, 12) * 16.66667) ;                   % duration of fixation (ms)
               FixationReport(entry, 19) = str2double(SUBJECT);
               
               if  ~(FixationReport(entry, 12) > 2 && FixationReport(entry, 12) <= 280) % remove this fixation from the report (as its too short or too long) 
                   entry = entry - 1;                                       
                   fixation_number = fixation_number -1;                  
                   trial_fixation  = trial_fixation  -1;
               end
               
           end
           
       end    
       eTag = 0;
       
   end
end


%********************************************************************
% Report only the required elements of the report and convert to .csv
%********************************************************************
subjectID(1:length(FixationReport), 1) = SUBJECT;

FixationReportConsise = [ subjectID  FixationReport(:,1) FixationReport(:,2) FixationReport(:,3) FixationReport(:,7) FixationReport(:,9) ...
    FixationReport(:,13) FixationReport(:,14) FixationReport(:,16) FixationReport(:,17) FixationReport(:,18)];


outputCSV = sprintf('FixationData_S_%s.csv', num2str(SUBJECT));
xlswrite(outputCSV, FixationReportConsise);













