%function WordPower_Louah
%global variables 
global EXPERIMENT;
global BACKCOLOR;
global EXPWIN;
global KEYBOARD;
global SUBJECT;
global WHITE 
global winWidth
global winHeight
global Calib 

eye_tracker = 1; %CHANGE TO 1 TO USE THE EYE TRACKER

%Include folders
addpath('eye_tracking_functions');
addpath('eye_tracking_functions/tetio');


%SET global parameters
EXPERIMENT='WordPower'; %the name of the experiment
SUBJECT=0; %default subject number
BACKCOLOR=[153 153 153]; %grey
KEYBOARD=1; %the first keyboard is used if more available


if (SUBJECT == 0) || isempty(SUBJECT)
        fprintf('\n');
        SUBJECT=input('Subject number: ');
        fprintf('\n');
end

s=sprintf('%5.3f',SUBJECT/1000.);
    if isempty(EXPERIMENT)
        EXPERIMENT='TEST';
    end

fprintf('\n****************************************\nExperiment %s\n',EXPERIMENT);
fprintf('%s\n',datestr(now));
fprintf('Subject %s\n\n',s(3:5));

%
%Follow the next steps:
%   1 - Sets up Psychtoolbox
%   2 - Connects to the eye tracker
%   3 - Calibrates participant
%   4 - Present stimuli and collect eye tracking data
%   5 - Saves raw Eye Tracking data into a csv file
%   6 - Plots the Eye Tracking data of the two first stimulus 


% *************************************************************************
%
% Initialization and connection to the Tobii Eye-tracker
%
% *************************************************************************
if eye_tracker 
    eyetrackerhost = 'TX120-204-10300441';  
    [ET, eye_tracker] = connect_to_eye_tracker(eye_tracker, eyetrackerhost, num2str(SUBJECT)); 
end
%Create .csv file with stimulus info
time = datestr(now, 'HH_MM_SS');
ET.stimulusInfofileName = strcat(pwd, '/results/StimulusInfo_',num2str(SUBJECT), '_', date,'_',time,'.csv');
stimulusInfoHeader = ['Subject ' 'Trial ' 'Condition ' 'Picture ' 'Object ' 'Fix_Cross_Onset ' 'Sound_Onset ' 'Sound_Offset ' 'Picture_Onset ' 'trial_paused '];
dlmwrite(ET.stimulusInfofileName, stimulusInfoHeader,'');
%**************************************************************************
%
% START PSYCHTOOLBOX SETTINGS
%
%**************************************************************************


%INIZIALIZE sound driver for high-precision, low-latency, multi-channel
%sound playback
InitializePsychSound;
PsychSettings; 
   

%CALIBRATE
if eye_tracker
    SetCalibParams; 
    ET.srate = tetio_getFrameRate;
    %Calibrate
    [pts,iter,calibTime, ET, eye_tracker, calibarray] = HandleCalibWorkflow(Calib,ET, EXPWIN, BACKCOLOR, eye_tracker); 
    if ~isempty(calibarray)
        imwrite(calibarray, [pwd '/results/Calib_results_' num2str(SUBJECT) '.jpg']);
    end
end



%**************************************************************************
%
% Present the stimuli and collect eye tracking data
%
%**************************************************************************

%
% INSTRUCTIONS
%
% READ the presentation screen size
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


textSize = 15;
Screen('FillRect', EXPWIN, BACKCOLOR);
Screen('TextSize', EXPWIN , textSize);
Screen('DrawText', EXPWIN, 'Press any key to begin', winWidth/2-100, winHeight/2, WHITE);
Screen(EXPWIN, 'Flip');


% while 1 %Wait until a key is pressed
%     pause (0.1);
%     keyIsDown = KbCheck;
%     if keyIsDown
%         break;% Begin with the experiment
%     end
% end
    
%**************************************************************************
%
% Start tracking
%
%**************************************************************************
if eye_tracker
    tetio_startTracking; 
end

    


%% Config Variables
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% EXPERIMENT variables
SUBJECT=1;
trialN = 96; % number of trials for the entire experiment.

% set BACKGROUND to grey
BACKCOLOR=[153 153 153];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Read in textures and images at the same time

fprintf(['C:\Users\Eye tracker\Documents\MATLAB\Louah\eye_tracking_functions\tetio\Sounds']);
pahaMatrix = pahamaker([pwd '\Sounds'],0);                                       
soundMatrix = wavemaker([pwd '\Sounds'],0);                                      

imageMatrix = facetexture([pwd '\Images'],0,EXPWIN);
%FigHandle=figure('Position', [100,100,900,900]);


orderMatrix = zeros(24,2);
countMe = 0;
for i=1:2:24 %length(pahaMatrix)
    countMe = countMe +1;
    for j=1:2
        orderMatrix(i,1) = countMe;
        orderMatrix(i+1,1) = countMe;
    end 
     
end
countMe = 1;
for k=1:4:24
    orderMatrix(k,2) = countMe;
    orderMatrix(k+1,2) = countMe+1;
    orderMatrix(k+2,2) = countMe;
    orderMatrix(k+3,2) = countMe+1;
    countMe = countMe +2;   
end
orderMatrix = [orderMatrix ; orderMatrix ; orderMatrix ; orderMatrix];
% ************* Generate fresh Random Number ******************
seed = ClockRandSeed; %SET the random seed from the time-of-day clock.
randomOrder1 = randperm(trialN);% Create a random order of all 24 trials
randomOrder2 = randperm(trialN);% Create a random order of all 24 trials
randomOrder3 = randperm(trialN);% Create a random order of all 24 trials
randomOrder4 = randperm(trialN);% Create a random order of all 24 trials

randomOrder = [randomOrder1 ; randomOrder2 ; randomOrder3 ; randomOrder4];

%SET objects size to fit the size of the presentation screen
%Stimuli must to be all of the same size
% FitScreen=[0 0 NewPicWidth NewPicHeight]; %SET the array with presentation parameters

%Variable to save the stimulus information
stimulusinfo.subject = zeros(trialN,1);
stimulusinfo.trial = zeros(trialN,1);
stimulusinfo.condition = zeros(trialN,1);
stimulusinfo.picture = zeros(trialN,1);
stimulusinfo.object = zeros(trialN,1);
stimulusinfo.onsettime = zeros(trialN,5); %5 columns: Cross onset time, Sound onset time, Sound offset time, Picture onset time, pause? 0 if trial was not paused and 1 if it was paused.

% **************LOOP through trials *****************************

for trial=1:trialN

    stimulusinfo.subject(trial, 1) = SUBJECT;
    stimulusinfo.trial(trial, 1) = trial;
    stimulusinfo.condition(trial, 1) = randomOrder(trial); % tell what condition is being extracted based on random order
    stimulusinfo.picture(trial, 1) = orderMatrix(randomOrder(trial),1); % extract image based on random order from orderMatrix
    stimulusinfo.object(trial, 1) = orderMatrix(randomOrder(trial),2); % extract word/sound based on random order from orderMatrix
    fprintf('trial %i    ', trial);
    fprintf(' %i ', randomOrder(trial));
    fprintf(' %i ', orderMatrix(randomOrder(trial),1));    
    fprintf(' %i \n', orderMatrix(randomOrder(trial),2));
    
    % 1) fixation point 
    % 2) sound with blank screen : an audio is played (word or sound)
    % 4) 1 second after the end of the spoken word an image appear (isi = 1 sec)
   

    %Fire Fixation and show for 1 seconds
    textSize = 300;
    Screen('FillRect', EXPWIN, BACKCOLOR);
    Screen('TextSize', EXPWIN , textSize);
    Screen('DrawText', EXPWIN, '+', winWidth/2 - 150, winHeight/2 - 150, WHITE);
    Screen('DrawingFinished',EXPWIN);
    onsettime = Screen(EXPWIN, 'Flip'); %time cross is shown
    stimulusinfo.onsettime(trial, 1) = onsettime;
    WaitSecs(1); 
    
    %% Play Audio with grey background 
%     Screen('FillRect',EXPWIN,BACKCOLOR); %replace background screen
%     Screen('DrawingFinished',EXPWIN);
%     onsettime;
    %Screen('Flip',EXPWIN); % Grey background while playing the sound 
    PsychPortAudio('FillBuffer', pahaMatrix(stimulusinfo.object(trial,1)), soundMatrix{1,stimulusinfo.object(trial,1)});

    PsychPortAudio('Start', pahaMatrix(stimulusinfo.object(trial,1)),1 , 0 ,0);
    stimulusinfo.onsettime(trial, 2) = GetSecs(); %time the sound starts playing
    [startTime, endPositionsec, xruns, soundOffset] = PsychPortAudio('Stop', pahaMatrix(stimulusinfo.object(trial,1)), 1,0,1);
    while 1
        status = PsychPortAudio('GetStatus', pahaMatrix(stimulusinfo.object(trial,1)));
        if status.Active
            WaitSecs(0.1);
        else
            break
        end
    end

    stimulusinfo.onsettime(trial, 3) = GetSecs();
    picture_onset = GetSecs()+ 1; %time the sound stops playing

    %% Object Picture
    Screen('DrawTexture',EXPWIN, imageMatrix(stimulusinfo.picture(trial,1)));
    Screen('DrawingFinished',EXPWIN);
    onsetTime = Screen('Flip',EXPWIN, picture_onset);            
    stimulusinfo.onsettime(trial, 4) = onsetTime; %Time the picture is shown

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%           

    % wait for 5 seconds & allow operator to stall the experement
    % if required
    Press = false; 

    TwoSecondWait = GetSecs() + 2; % 2.5 seconds because we will spend 0.5 seconds collecting eye tracking data 
    while TwoSecondWait > GetSecs() % show the picture for 2 seconds and check whether we want to pause the presentation
       Press = KbCheck;               % Any operator key
       WaitSecs(0.2);
       if Press
            fprintf('PAUSE \n');
            WaitSecs(0.3);
            stimulusinfo.onsettime (trial, 5) = 1;
            while 1
                Press = KbCheck;               % Any operator key
                WaitSecs(0.2);
                if Press
                    fprintf('CONTINUE \n');
                    Press = false;
                    break
                end
            end        
       end
    end

    if eye_tracker
        %Collect eye tracking data after the trial
        % Convert to csv file each trial data collected
        [ET] = DataCollect(ET); 
    end
    %Save stimulus info variables
    trialInfo = [stimulusinfo.subject(trial,1) stimulusinfo.trial(trial,1) stimulusinfo.condition(trial,1) stimulusinfo.picture(trial,1) stimulusinfo.object(trial,1) stimulusinfo.onsettime(trial, :)];
    dlmwrite(ET.stimulusInfofileName, trialInfo,'precision',16,'delimiter','\t','-append');

    
end        

%**********************************************************************
%
%  STOP TRACKING AND Close
%
%**********************************************************************
if eye_tracker
    tetio_stopTracking; 
    tetio_disconnectTracker; 
    tetio_cleanUp;
    %save (['results/matlab_files_' num2str(SUBJECT)], 'ET', 'verTimes', 'data', 'windowRect')
     save (['results/matlab_files_' num2str(SUBJECT)], 'ET');
end

%Close Psychtoobox screen, set priotity to 0 and show the cursor
Screen('CloseAll');
Priority(0);
ShowCursor;


disp('Program finished.');
%return





